/*
** Module   : CommonSession.java
** Abstract : Define all APIs available via the SESSION system handle.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CS  20130211 Moved first/last procedure attribute to a separate interface. 
*/

package com.goldencode.p2j.util;

/**
 * Exposes the following methods or attributes from the SYSTEM handle:
 * <table>
 * <tr>
 *    <th>SESSION attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>BATCH-MODE</td>
 *    <td>{@link #isBatchMode()}</td>
 * </tr>
 * <tr>
 *    <td>CHARSET</td>
 *    <td>{@link #getCharset()}</td>
 * </tr>
 * <tr>
 *    <td>DATE-FORMAT</td>
 *    <td>{@link #getDateOrder()}, {@link #setDateOrder(character)} and 
 *        {@link #setDateOrder(String)}
 *    </td>
 * </tr>
 * <tr>
 *    <td>DISPLAY-TYPE</td>
 *    <td>{@link #getDisplayType()}</td>
 * </tr>
 * <tr>
 *    <td>HEIGHT-CHARS</td>
 *    <td>emitted as getHeightChars and setHeightChars, not implemented</td>
 * </tr>
 * <tr>
 *    <td>HEIGHT-PIXELS</td>
 *    <td>{@link #getHeightPixels()} and setHeightPixels</td>
 * </tr>
 * <tr>
 *    <td>TEMP-DIRECTORY</td>
 *    <td>{@link #getTempDirectory()}</td>
 * </tr>
 * <tr>
 *    <td>WIDTH-CHARS</td>
 *    <td>emitted as getWidthChars and setWidthChars, not implemented</td>
 * </tr>
 * <tr>
 *    <td>WIDTH-PIXELS</td>
 *    <td>{@link #getWidthPixels()} and setWidthPixels</td>
 * </tr>
 * <tr>
 *    <td>LAST-PROCEDURE</td>
 *    <td>{@link #lastProcedure()}</td>
 * </tr>
 * <tr>
 *    <td>FIRST-PROCEDURE</td>
 *    <td>{@link #firstProcedure()}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapSession(handle)} API, which, depending on the 
 * handle's state, it either returns an {@link SessionUtils#asHandle()} 
 * instance or a dynamic proxy using {@link handle#invalidAttrAccessProxy}. 
 */
public interface CommonSession
extends CommonProcedure,
        CommonHandle,
        FirstLastProcedureAttribute
{
   /**
    * Reports if the current context is running in batch mode.
    * <p>
    * The value returned may have been found via a search algorithm that
    * is account (user or process) specific or group specific
    * within the current server or a global default for all servers. 
    * <p>
    * The implementation iteratively looks up the directory node under:
    * /server/&lt;serverID&gt;/runtime/&lt;account_or_group&gt;/batchMode
    * <p>
    * If no user/process or group nodes are present, then this is checked:
    * /server/&lt;serverID&gt;/runtime/default/batchMode
    * <p>
    * If no /server/&lt;serverID&gt;/runtime node exists, this is checked
    * (it is the global default area for all servers):
    * /server/default/runtime/&lt;account_or_group&gt;/batchMode
    * <p>
    * Finally, if no user/process or group nodes are present in the global
    * default area, then this is checked:
    * /server/default/runtime/default/batchMode
    * <p>
    * If no value is found via this lookup, then the default value of
    * <code>false</code> will be returned.
    *
    * @return   <code>true</code> if the current context is in batch mode.
    */
   public logical isBatchMode();
   
   /**
    * Reports the default character set name. Since the Progress default
    * charset is ISO-8859-1, when the JVM reports that UTF-8 is the default,
    * this method will return the string "ISO-8859-1" instead.
    * <p>
    * The assumption (possibly not always correct) is that UTF-8 is not a
    * valid Progress charset and instead, to provide a proper default, we
    * must override with a compatible charset. Note that if the default
    * charset name is not UTF-8, it is assumed that the charset has been
    * overriden already (at JVM startup).  In that case, the default is
    * returned unchanged.
    *
    * @return   A default charset name.
    */
   public character getCharset();

   /** 
    * Gets the order of date components for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components is a 3 character string using
    * "M", "D" and "Y" once each.  The order from index 0 to index 2
    * represents the left to right ordering of the components. So the leftmost
    * date component will be defined by the character at index position 0.
    * <p>
    * The value accessed is specific to the current user's context.
    *
    * @return   Defines the order of date components for conversion to/from
    *           strings.
    */
   public character getDateOrder();
   
   /** 
    * Sets the order of date components for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components as a 3 character string using
    * "M", "D" and "Y" once each.  The order from index 0 to index 2
    * represents the left to right ordering of the components. So the leftmost
    * date component will be defined by the character at index position 0.
    * <p>
    * The value set is specific to the current user's context.
    *
    * @param    dateFormat
    *           Defines the order of date components for conversion to/from
    *           strings.
    */
   public void setDateOrder(String dateFormat);
   
   /** 
    * Sets the order of date components for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components as a 3 character string using
    * "M", "D" and "Y" once each.  The order from index 0 to index 2
    * represents the left to right ordering of the components. So the leftmost
    * date component will be defined by the character at index position 0.
    * <p>
    * The value set is specific to the current user's context.
    *
    * @param    dateFormat
    *           Defines the order of date components for conversion to/from
    *           strings.
    */
   public void setDateOrder(character dateFormat);
   
   /**
    * Gets the display type (GUI or TTY).  Corresponds with the
    * <code>SESSION:DISPLAY-TYPE</code> system handle attribute.
    * 
    * @return  Display type.  Currently, only TTY is supported.
    */
   public character getDisplayType();

   /**
    * Get the width in pixels of the display.  Corresponds with the
    * <code>SESSION:WIDTH-PIXELS</code> system handle attribute.
    * 
    * @return  Display width in pixels.
    */
   public integer getWidthPixels();

   /**
    * Get the height in pixels of the display.  Corresponds with the
    * <code>SESSION:HEIGHT-PIXELS</code> system handle attribute.
    * 
    * @return  Display height in pixels.
    */
   public integer getHeightPixels();
   
   /**
    * Gets the name of the directory where temporary files are stored.
    * Corresponds with with the <code>SESSION:TEMP-DIRECTORY</code> system
    * handle attribute.
    * 
    * @return  Name of temporary directory.
    */
   public character getTempDirectory();
   
   /**
    * Method that specifies the remote procedures the client can execute 
    * from the AppServer session
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be
    *           exported.
    *         
    * @return   <true> if REMOTE attribute is true and operation succeeded or
    *           <false> otherwise
    */
   public logical export(String attrList);
   
   /**
    * Method that specifies the remote procedures the client can execute 
    * from the AppServer session
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be
    *           exported.
    *         
    * @return   <true> if REMOTE attribute is true and operation succeeded or
    *           <false> otherwise
    */
   public logical export(character attrList);
   
   /**
    * Method implementation for getting the SERVER-OPERATING-MODE attribute of
    * the session handle.
    * <p>
    * Possible values:
    * <ul>
    *    <li>State-reset
    *    <li>State-aware
    *    <li>Stateless
    *    <li>State-free
    * </ul>
    * 
    * @return   the corresponding SERVER-OPERATING-MODE attribute of the session 
    *           handle.
    */
   public character getServerOperatingMode();
   
   /**
    * Indicates if the session is an AppServer session or not.
    * 
    * @return   <code>true</code> if the current session is an AppServer session
    *           or false otherwise.
    */
   public logical isRemote();
}